<?php

/**
 * @file
 * Hooks provided by the Node Link Style.
 */

/**
 * @addtogroup hooks
 * @{
 */

/**
 * Define all node links that are able to be styled.
 *
 * @return array
 *   An associative array keyed by unique name of the node links. Each link is
 *   an associative array containing:
 *   - label: The human readable name of the link displayed in the admin UI.
 */
function hook_node_link_style_info() {
  // Single link provided by a module.
  $links['blog_usernames_blog'] = array(
    'label' => t('Blog author'),
  );
  // A group of links, "__" is a wildcard that will match any suffix.
  $links['taxonomy_term__'] = array(
    'label' => t('Taxonomy terms'),
    'group' => TRUE,
  );

  return $links;
}

/**
 * Default style configurations.
 *
 * @reuturn
 *   An associative array keyed by unique name of the node link style. Each
 *   item in the array is an object containing the configuration definitions.
 */
function hook_node_link_style_default_styles() {
  $styles = array();

  $style = new stdClass;
  $style->disabled = FALSE;
  $style->api_version = 1;
  $style->name = 'test_config';
  $style->settings = array(
    'links' => array(
      'comment_add' => array(
        'show' => 1,
        'element' => 'div',
        'class' => '',
        'apply_attributes' => 0,
        'info' => array(
          'label' => 'Add comment',
        ),
      ),
      'node_read_more' => array(
        'show' => TRUE,
      ),
    ),
  );
  $styles['test_config'] = $style;

  return $styles;
}

/**
 * @} End of "addtogroup hooks".
 */
