<?php

/**
 * @file
 * Theme hooks provided by the Node Link Display module.
 */

/**
 * Returns HTML for a styled node link.
 *
 * @param $name
 *   The name of the link as defined in the hook.
 * @param $link
 *   An array containing the return of the hook_link() implementation.
 * @param $config
 *   An array containing the configurations.
 *
 * @return
 *   A string containing the HTML for the node link.
 *
 * @ingroup themeable
 */
function theme_node_link_style_link($name, $link, $config) {
  $output = '';

  // Open the wrapper element.
  if (!empty($config['element'])) {

    // Builds wrapper element attributes.
    if (empty($config['apply_attributes'])) {
      $class = $name;
      if (!empty($config['class'])) {
        $class .= ' ' . $config['class'];
      }
      $attributes = drupal_attributes(array('class' => $class));
    }
    else {
      $attributes = '';
      if (isset($link['attributes'])) {
        if (empty($link['attributes']['class'])) {
          $link['attributes']['class'] = $name;
        }
        else {
          $link['attributes']['class'] .= ' ' . $name;
        }
        $attributes = drupal_attributes($link['attributes']);
      }
    }

    // Initializes the element
    $element = check_plain($config['element']);
    $output .= '<' . $element . $attributes . '>';
  }

  // Add the link or text.
  if (isset($link['href'])) {
    $output .= l($link['title'], $link['href'], $link);
  }
  elseif (!empty($link['title'])) {
    if (empty($link['html'])) {
      $link['title'] = check_plain($link['title']);
    }
    $output .= $link['title'];
  }

  // Close the wrapper element.
  if (!empty($config['element'])) {
    $output .= '</' . $element . '>';
  }

  // Returns themed link.
  return $output;
}

/**
 * Returns HTML for a styled group of node links.
 *
 * @param $name
 *   The name of the link as defined in the hook.
 * @param $links
 *   An array containing the return of the hook_link() implementation.
 * @param $config
 *   An array containing the configurations.
 *
 * @return
 *   A string containing the HTML for the node link.
 *
 * @ingroup themeable
 */
function theme_node_link_style_group($links, $config) {
  global $language;
  $output = '';

  // Gets link count, gets list elements and attributes.
  $num_links = count($links);
  $list_element = check_plain($config['list_element']);
  $list_attributes = drupal_attributes(array('class' => $config['list_class']));

  // Gets item element based on list type.
  switch ($list_element) {
    case 'ul':
    case 'ol':
      $item_element = 'li';
      break;
    default:
      $item_element = 'dt';
      break;
  }

  // Builds the list.
  if ($num_links) {
    $output .= '<' . $list_element . $list_attributes . '>';

    $i = 1;
    foreach ($links as $name => $link) {
      $class = $name;

      // Add first, last and active classes.
      if ($i == 1) {
        $class .= ' first';
      }
      if ($i == $num_links) {
        $class .= ' last';
      }
      if (isset($link['href']) && ($link['href'] == $_GET['q'] || ($link['href'] == '<front>' && drupal_is_front_page()))
          && (empty($link['language']) || $link['language']->language == $language->language)) {
        $class .= ' active';
      }

      // Builds list item via theme_node_link_style_link().
      $output .= '<' . $item_element . drupal_attributes(array('class' => $class)) . '>';
      $output .= theme('node_link_style_link', $name, $link, $config);
      $output .= "</$item_element>\n";

      // Increments row count.
      $i++;
    }

    // Finalizes the list.
    $output .= '</' . $list_element . '>';
  }

  // Returnes the themed group.
  return $output;
}
