<?php

/**
 * @file
 * Context reaction plugin for the Node Link Style module.
 */

/**
 * Expose menu items as context reactions.
 */
class node_link_style_reaction_apply extends context_reaction {

  /**
   * Implements context_reaction::options_form().
   */
  function options_form($context) {
    ctools_include('export');

    // Gathers options.
    $options = array('0' => t('- None -'));
    foreach ((array) ctools_export_crud_load_all('node_link_style') as $name => $config) {
      $options[$name] = check_plain($name);
    }

    // Returns form element.
    return array(
      '#type' => 'select',
      '#title' => t('Apple node link style'),
      '#description' => t('Select the style configuration to apply to the node links.'),
      '#options' => $options,
      '#default_value' => $this->fetch_from_context($context),
    );
  }

  /**
   * Execute the reaction of theming the node links.
   *
   * @param $links
   *   An array of node links being styled.
   *
   * @return
   *   The themed links.
   */
  function execute($links) {
    foreach ($this->get_contexts() as $context) {
      if (isset($context->reactions[$this->plugin])) {
        if ($style = node_link_style_item_load($context->reactions[$this->plugin])) {
          return $this->style_links($links, $style);
        }
      }
    }
  }

  /**
   * Applies the configured styles to the node links.
   *
   * @param $links
   *   An array of node links being styled.
   * @param $style
   *   The style configuration settings.
   *
   * @return
   *   The themed links.
   */
  function style_links($links, $style) {
    $output = '';

    // Loops over configurations, styles appropriate links.
    foreach ($style->settings['links'] as $link => $config) {
      if (empty($config['info']['group'])) {
        // Theme the link if it exists and is configured to be displayed.
        if (isset($links[$link]) && !empty($config['show'])) {
          $output .= theme('node_link_style_link', $link, $links[$link], $config);
        }
        // Unset the link so we don't theme it again.
        unset($links[$link]);
      }
      else {
        // Initializes container for links in the group.
        $group = array();

        // Matches links based on pattern.
        $pattern = '/^' . str_replace('__', '_.*', $link) . '$/';
        foreach ($links as $link_name => $node_link) {
          if (preg_match($pattern, $link_name)) {
            $group[$link_name] = $links[$link_name];
            unset($links[$link_name]);
          }
        }

        // Themes the group.
        if ($group) {
          $output .= theme('node_link_style_group', $group, $config);
        }
      }
    }

    // Themes the remaining node links in the normal way.
    if (!empty($links)) {
      $output .= theme('links', $links, array('class' => 'links inline'));
    }

    // Returns the new node links.
    return $output;
  }
}
