<?php

/**
 * @file
 * Export UI display customizations.
 */

/**
 * CTools export UI extending class.
 */
class node_link_style_export_ui extends ctools_export_ui {

  /**
   * Implements ctools_export_ui::edit_page().
   *
   * This is hacky, but since CTools Export UI uses drupal_goto() we have to
   * effectively change the plugin to modify the redirect path dynamically.
   */
  function edit_execute_form(&$form_state) {
    $output = parent::edit_execute_form($form_state);
    if (!empty($form_state['executed'])) {
      if (t('Add link') == $form_state['clicked_button']['#value']) {
        $path = 'admin/settings/node_link_style/list/' . $form_state['item']->name . '/edit';
        $this->plugin['redirect']['edit'] = $path;
      }
    }
    return $output;
  }
}

/**
 * Function used by uasort to sort structured arrays by weight.
 *
 * @see http://api.drupal.org/api/drupal/includes--common.inc/function/drupal_sort_weight/7
 */
function node_link_style_sort_weight($a, $b) {
  $a_weight = (is_array($a) && isset($a['weight'])) ? $a['weight'] : 0;
  $b_weight = (is_array($b) && isset($b['weight'])) ? $b['weight'] : 0;
  if ($a_weight == $b_weight) {
    return 0;
  }
  return ($a_weight < $b_weight) ? -1 : 1;
}

/**
 * Define the preset add/edit form.
 *
 * @see node_link_style_settings_form_submit()
 * @ingroup forms
 */
function node_link_style_settings_form(&$form, &$form_state) {

  // Initializes the item.
  $item = &$form_state['item'];
  if (empty($item->settings)) {
    $item->settings = array('links' => array());
  }
  uasort($item->settings['links'], 'node_link_style_sort_weight');

  // Deletes item from the list.
  if (!empty($_GET['delete'])) {
    unset($item->settings['links'][$_GET['delete']]);
    ctools_export_crud_save('node_link_style', $item);
    $path = 'admin/settings/node_link_style/list/' . $item->name . '/edit';
    drupal_goto($path);
  }

  // Adds CSS and JavaScript.
  $module_path = drupal_get_path('module', 'node_link_style');
  drupal_add_css($module_path . '/node_link_style.css');
  drupal_add_js($module_path . '/node_link_style.js');

  // Is Vertical Tabs installed?
  $vertical_tabs = module_exists('vertical_tabs');

  // Initializes the weight of the info.
  $form['info']['#weight'] = -30;

  // If we are creating the configuration, only display the basic config items.
  if (empty($form['info']['name']['#default_value'])) {
    return;
  }

  $form['links'] = array(
    '#type' => 'fieldset',
    '#title' => t('Available node links'),
    '#weight' => -30,
  );

  $form['link_weight_label'] = array(
    '#type' => 'item',
    '#value' => t('Link order'),
    '#access' => !empty($item->settings['links']),
    '#prefix' => '<h3>',
    '#suffix' => '</h3>',
    '#weight' => -25,
  );

  $form['link_weight'] = array(
    '#theme' => 'node_link_style_weight_table',
    '#weight' => -20,
    '#access' => !empty($item->settings['links']),
    'weight' => array('#tree' => TRUE),
  );

  $form['link_settings_label'] = array(
    '#type' => 'item',
    '#value' => t('Link settings'),
    '#access' => !empty($item->settings['links']),
    '#prefix' => '<h3>',
    '#suffix' => '</h3>',
    '#weight' => -15,
  );

  $form['link_settings'] = array(
    '#tree' => TRUE,
    '#weight' => -10,
    '#access' => !empty($item->settings['links']),
  );
  if ($vertical_tabs) {
    $form['link_settings']['#pre_render'][] = 'vertical_tabs_form_pre_render';
  }

  // Gets information from hooks, stores return from all hooks.
  $options = $style_info = array();
  $files = module_rebuild_cache();
  foreach (module_implements('node_link_style_info') as $module) {
    if ($return = module_invoke($module, 'node_link_style_info')) {
      // Captures human readable name of the module.
      $module_name = $files[$module]->info['name'];

      // Adds item to list if not already styled.
      foreach ($return as $link => $info) {
        $style_info[$link] = $info;
        if (!isset($item->settings['links'][$link])) {
          $options[$module_name][$link] = check_plain($info['label']);
        }
      }

      // Sorts sub-section by display name if there are values.
      if (!empty($options[$module_name])) {
        asort($options[$module_name]);
      }
    }
  }

  // Sort $options by module name.
  ksort($options);

  $form['links']['link'] = array(
    '#type' => 'select',
    '#title' => t('Style link'),
    '#options' => $options,
    '#description' => t('Add a link or group of links to the custom style list.'),
  );

  $form['links']['add_item'] = array(
    '#type' => 'submit',
    '#value' => t('Add link'),
  );

  // Wrapper elements
  $elements = array(
    '0' => t('- None -'),
    'div' => 'DIV',
    'span' => 'SPAN',
    'h1' => 'H1',
    'h2' => 'H2',
    'h3' => 'H3',
    'h4' => 'H4',
    'h5' => 'H5',
    'h6' => 'H6',
    'p' => 'P',
    'strong' => 'STRONG',
    'em' => 'EM',
  );

  // Iterates over each link, adds individual settings.
  foreach ($item->settings['links'] as $link => $settings) {

    // Adds label to the weight table.
    $form['link_weight']['label'][$link] = array(
      '#type' => 'markup',
      '#value' => check_plain($style_info[$link]['label']),
    );

    // Link to remove the item.
    $path = 'admin/settings/node_link_style/item/' . $form_state['item']->name . '/delete/' . $link;
    $form['link_weight']['remove'][$link] = array(
      '#type' => 'markup',
      '#value' => l(t('Remove styled link'), $path),
    );

    // Adds item to weight table.
    $form['link_weight']['weight'][$link] = array(
      '#type' => 'weight',
      '#title' => '',
      '#delta' => 50,
      '#default_value' => $settings['weight'],
      '#attributes' => array('class' => 'node-link-style-weight'),
    );

    // Merges in defaults.
    $settings += array(
      'weight' => 0,
      'show' => TRUE,
      'group_element' => 0,
      'group_class' => '',
      'list_element' => 'ul',
      'list_class' => 'links inline',
      'element' => 0,
      'class' => '',
      'apply_attributes' => FALSE,
    );

    // Adds a fieldset for each link.
    $form['link_settings'][$link] = array(
      '#type' => 'fieldset',
      '#title' => check_plain($style_info[$link]['label']),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    if ($vertical_tabs) {
      $form['link_settings']['#group'] = 'vertical_tabs';
    }

    $form['link_settings'][$link]['show'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show link'),
      '#default_value' => $settings['show'],
    );

    if (!empty($style_info[$link]['group'])) {
      $form['link_settings'][$link]['group_element'] = array(
        '#type' => 'select',
        '#title' => t('Group wrapper element'),
        '#options' => $elements,
        '#default_value' => $settings['group_element'],
      );

      $form['link_settings'][$link]['group_class'] = array(
        '#type' => 'textfield',
        '#title' => t('Group CSS class'),
        '#default_value' => $settings['group_class'],
      );

      $form['link_settings'][$link]['list_element'] = array(
        '#type' => 'select',
        '#title' => t('List type'),
        '#options' => array(
          'ul' => 'UL',
          'ol' => 'OL',
          'dl' => 'DL',
        ),
        '#default_value' => $settings['list_element'],
      );

      $form['link_settings'][$link]['list_class'] = array(
        '#type' => 'textfield',
        '#title' => t('List CSS class'),
        '#default_value' => $settings['list_class'],
      );
    }

    $form['link_settings'][$link]['element'] = array(
      '#type' => 'select',
      '#title' => t('Link wrapper element'),
      '#options' => $elements,
      '#default_value' => $settings['element'],
    );

    $form['link_settings'][$link]['class'] = array(
      '#type' => 'textfield',
      '#title' => t('Link CSS class'),
      '#default_value' => $settings['class'],
    );

    $form['link_settings'][$link]['apply_attributes'] = array(
      '#type' => 'checkbox',
      '#title' => t('Apply link attributes to wrapper element'),
      '#default_value' => $settings['inherit'],
      '#description' => t('This is useful when the node link is just text and expects that the attributes are applied to the wrapper element.')
    );

    $form['link_settings'][$link]['info'] = array(
      '#type' => 'value',
      '#value' => $style_info[$link],
    );
  }
}

/**
 * Form submission handler for facetapi_realm_settings_form().
 */
function node_link_style_settings_form_submit($form, &$form_state) {
  $item = &$form_state['item'];

  // Handles adding another item to to table.
  if (t('Add link') == $form_state['clicked_button']['#value']) {
    $link = $form_state['values']['link'];
    $item->settings['links'][$link] = array('show' => TRUE);
  }

  // Handles saving the settings.
  if (t('Save') == $form_state['clicked_button']['#value']) {
    if (isset($form_state['values']['link_settings'])) {
      uasort($form_state['values']['link_settings'], 'node_link_style_sort_weight');
      $item->settings['links'] = $form_state['values']['link_settings'];
      foreach ($form_state['values']['weight'] as $link => $weight) {
        $item->settings['links'][$link]['weight'] = $weight;
      }
    }
    else {
      $item->settings['links'] = array();
    }
  }
}

/**
 * Themes the facet form into a draggable table.
 *
 * @param $form
 *   A FAPI array containing a fieldset.
 *
 * @ingroup themable
 */
function theme_node_link_style_weight_table($form) {
  $output = '';

  // Makes the table draggable.
  $table_id = 'node-link-style-weight-table';
  drupal_add_tabledrag($table_id, 'order', 'sibling', 'node-link-style-weight');

  // Adds each link as a row in the table.
  $rows = array();
  foreach (element_children($form['weight']) as $link) {
    $rows[] = array(
      'class' => 'draggable',
      'data' => array(
        array(
          'data' => drupal_render($form['label'][$link]),
        ),
        array(
          'data' => drupal_render($form['remove'][$link]),
        ),
        array(
          'data' => drupal_render($form['weight'][$link]),
          'class' => 'dropbox'
        ),
      ),
    );
  }

  // formats the table for the facets, the table will have dragable rows
  $form['#value'] .= theme(
    'table', array(), $rows, array('id' => $table_id)
  );

  $output .= drupal_render($form);
  return $output;
}

////
////
//// Core module hook implementations.
////
////

/**
 * Implements hook_node_link_style().
 *
 * Blog module's implementation of this hook.
 */
function blog_node_link_style_info() {
  $links['blog_usernames_blog'] = array(
    'label' => t('Blog author'),
  );
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Book module's implementation of this hook.
 */
function book_node_link_style_info() {
  $links['book_add_child'] = array(
    'label' => t('Add child page to the book node'),
  );
  $links['book_printer'] = array(
    'label' => t('Printer-friendly version of book node'),
  );
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Comment module's implementation of this hook.
 */
function comment_node_link_style_info() {
  $links['comment_comments'] = array(
    'label' => t('Number of comments'),
  );
  $links['comment_new_comments'] = array(
    'label' => t('New comments'),
  );
  $links['comment_add'] = array(
    'label' => t('Add comment'),
  );
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Node module's implementation of this hook.
 */
function node_node_link_style_info() {
  $links['node_read_more'] = array(
    'label' => t('Read more link'),
  );
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Statistics module's implementation of this hook.
 */
function statistics_node_link_style_info() {
  $links['statistics_counter'] = array(
    'label' => t('Node view statistics'),
  );
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Translation module's implementation of this hook.
 */
function translation_node_link_style_info() {
  $links['node_translation__'] = array(
    'label' => t('Node translated in another language'),
    'group' => TRUE,
  );
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Upload module's implementation of this hook.
 */
function upload_node_link_style_info() {
  $links['upload_attachments'] = array(
    'label' => t('Uploaded files'),
  );
  return $links;
}

////
////
//// Contributed module hook implementations.
////
////

/**
 * Implements hook_node_link_style().
 *
 * Calendar module's implementation of this hook.
 */
function calendar_link_node_link_style_info() {
  $links['calendar_link'] = array(
    'label' => t('View calendar'),
  );
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Flag module's implementation of this hook.
 */
function flag_node_link_style_info() {
  $links = array();
  foreach (flag_get_flags('node') as $flag) {
    // Only get flags that are displayed on nodes.
    if ($flag->uses_hook_link(TRUE) || $flag->uses_hook_link(FALSE)) {
      $links['flag-'. $flag->name] = array(
        'label' => t('@title flag (@name)', array('@title' => $flag->title, '@name' => $flag->name)),
      );
    }
  }
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Image module's implementation of this hook.
 */
function image_node_link_style_info() {
  $links = array();
  foreach (image_get_sizes() as $key => $size) {
    if ($size['link']) {
      $links['image_size_' . $key] = array(
        'label' => t($size['label']),
      );
    }
  }
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Mollom module's implementation of this hook.
 */
function mollom_node_link_style_info() {
  $links['mollom_report'] = array(
    'label' => t('Report to Mollom'),
  );
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * Notifications UI module's implementation of this hook.
 */
function notifications_ui_node_link_style_info() {
  $links['notifications__'] = array(
    'label' => t('Subscription links'),
    'group' => TRUE,
  );
  return $links;
}

/**
 * Implements hook_node_link_style().
 *
 * User to User Points module's implementation of this hook.
 */
function user2userpoints_node_link_style_info() {
  $links['user2userpoints'] = array(
    'label' => t('Give !points', userpoints_translation()),
  );
  return $links;
}
